/** 
 *   vac - Communication routines for thermovac, penninvac and multivac devices
 *
 *   (c) Copyright 2006 Fernando P. Hauscarriaga 
 *   <fernandoph@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

#include "vac.h"

/*
 * vac_init - Open device
 */

int vac_init(char *name)
{
  int fd;
  struct termios newtio;
 
  fd = open(name, O_RDWR);

  memset(&newtio, 0, sizeof(newtio));
  newtio.c_cflag = (BAUDRATE|CS8|CREAD|CLOCAL|CSTOPB);
  newtio.c_iflag &= IXON;
  newtio.c_cc[VMIN]  = 1;
  newtio.c_cc[VTIME] = 0;
 
  tcsetattr(fd, TCSANOW, &newtio);
  
  return(fd);
}

/*
 * vac_sendcmd - Send command to the serial interface
 */

int vac_sendcmd (int fd, char *cmd)
{
  int resul;

  resul = write(fd, cmd, strlen(cmd));
  
  return resul;
}

/*
 * vac_write - Sends a command to the device, checks ACK/NAK and reports
 *                   error conditions in case of NAK
 *                   Arguments must be the file descriptor and 
 *                   a string ending with a CR (0x0D)
 */ 

int vac_write(int fd, char *cmd)
{
  int resul;
  char eri_cmd[6];
  char *buff;

  resul = vac_sendcmd (fd, cmd);

  if (resul < 0){
    printf("vac_write failed.\n");
    return -1;
  }
  else {
    if (check_ack(fd) < 0){
      buff = (unsigned char *) malloc(21);
      sprintf (eri_cmd, "eri r%c", CR);
      vac_sendcmd (fd, eri_cmd);
      vac_read (fd, buff);
      printf("%s\n", buff);
      free(buff);
      return -1;
    }
  }

  return(resul);
}
  
/*
 * vac_read - Reads vac device
 */

int vac_read (int fd, char *recv)
{
  int len = 0;
  int l;

  do{
    l = read (fd, recv+len, 1);
    if(*(recv+len) != LF)
      len += l;
  } while ( len == 0 || recv[len-1] != CR);
  return (l);
}

/*
 * check_ack - Checks for ACK char in serial device
 */

int check_ack (int fd)
{
  char recv[BUFF];

  vac_read (fd, recv);
 
  if (recv[0] == ACK)
    return 1;
  else if (recv[0] == NAK)
    return -1;
  return 0;
}

