/** 
 *   Configuration tools for VAC
 *
 *   (c) Copyright 2006 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   29/07/2006: Created.
 **/

#define _GNU_SOURCE


#include <stdio.h>
//#include <stdint.h>
//#include <limits.h>
//#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "my_stdio.h"
#include "parse_tools.h"

#include "vac.h"

#define MAGIC_TAGS "instrument"


int free_vac(struct vac *vac_conf)
{
	int j;
	struct vac *v = vac_conf;
	struct vac *vp;

	while(v != NULL) {
		vp = v;
		v = v->next;
		if (vp->name != NULL) free(vp->name);
		if (vp->device != NULL) free(vp->device);
		for(j=0;j<vp->sensor_n;j++)
			if (vp->sensor[j] != NULL) free(vp->sensor[j]);
		if (vp->sensor != NULL) free(vp->sensor);
		free(vp); 
	}

	return 0;
}


static struct vac *make_vac(void)
{
	struct vac *v;

	if ( (v = (struct vac *) malloc(sizeof(struct vac))) != NULL) {
		v->fd = -1;
		v->name = NULL;
		v->device = NULL;
		v->sensor_n = 0;
		v->sensor = NULL;
		v->next = NULL;
	}
	     
	return v;
}
	

int make_vac_from_conf(struct vac **vac_conf, struct configuration *conf)
{
	int instruments_n = -1;
	struct configuration *tag;
	int i, j, l;
	struct vac *v;
	char line[80];

	if ( *vac_conf != NULL) free_vac(*vac_conf);
	v = *vac_conf = make_vac();
	if ((tag = find_conf_tag(conf, MAGIC_TAGS".n")) != NULL) {
		instruments_n = atoi(tag->data);
		for(i = 0; i < instruments_n; i++) {
			sprintf(line, "%s.%d.name",MAGIC_TAGS, i);
			tag = find_conf_tag(conf, line);
			if ( tag != NULL) {
				l = strlen(tag->data) + 1;
				v->name = (char *) malloc(l);
				strncpy(v->name, tag->data, l);
			}
			sprintf(line, "%s.%d.device",MAGIC_TAGS, i);
			tag = find_conf_tag(conf, line);
			if ( tag != NULL) {
				l = strlen(tag->data) + 1;
				v->device = (char *) malloc(l);
				strncpy(v->device, tag->data, l);
			}
			sprintf(line, "%s.%d.sensors.n",MAGIC_TAGS, i);
			tag = find_conf_tag(conf, line);
			if ( tag != NULL) {
				v->sensor_n = atoi(tag->data);
				v->sensor = (char **) malloc(v->sensor_n);
				for (j=0; j < v->sensor_n; j++) {
					v->sensor[j] = NULL;
					sprintf(line, "%s.%d.sensors.%d",
						MAGIC_TAGS, i, j);
					tag = find_conf_tag(conf, line);
					if ( tag != NULL) {
						l = strlen(tag->data) + 1;
						v->sensor[j] = (char *) malloc(l);
						strncpy(v->sensor[j], tag->data, l);
					}
				}
			}
			v->next = make_vac();
			v = v->next;
		}		
	}

	return instruments_n;
}

/*
 * get_interval - Gets interval value from config file
 */

unsigned long get_interval (struct configuration *conf)
{
  struct configuration *tag;
  unsigned long seconds;

  seconds = 0;
  
  if ((tag = find_conf_tag(conf, "interval")) != NULL)
    return (atol(tag->data));
  
  if ((tag = find_conf_tag(conf, "seconds")) != NULL)
    seconds = atol(tag->data);
  seconds = seconds*1000000;
  return (seconds);
  
  return 0;
}
/*
char *get_note (struct configuration *conf)
{
  struct configuration *tag;

  if ((tag = find_conf_tag(conf, "note")) != NULL)
  return tag->data;
  }
*/
